/*
#define BUILD_LIBV_DLL
#define DLL_TARGET_DCC
*/

#include <math.h>
#include "vglext.c"
#include "nanoglut.c"
#include "nanoveclib.c"

static void upload()
{
	static GLfloat tex[] = {
		1,.5f,1,.6f,
		.3f,1,.2f,1,
		1,.2f,1,.3f,
		.6f,1,.5f,1
	};
	glTexImage2D(GL_TEXTURE_2D,0,1,4,4,0,GL_LUMINANCE,GL_FLOAT,tex);
	glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
}

static GLuint quad;

static void
draw_setup(GLsizei width, GLsizei height)
{
	GLfloat aspect = (GLfloat)width / (GLfloat)height;
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-100 * aspect,100 * aspect,-100,100,-100,100);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glActiveTextureARB(GL_TEXTURE0_ARB);
	upload();
	glEnable(GL_TEXTURE_2D);
	glActiveTextureARB(GL_TEXTURE1_ARB);
	upload();
	glEnable(GL_TEXTURE_2D);
	quad = glGenLists(1);
	glNewList(quad,GL_COMPILE);
		glColor3f(0.95f,0.8f,1);
		glBegin(GL_QUADS);
			/* Note: the texcoords are _idential_ */
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB,-1,-1);
			glMultiTexCoord2fARB(GL_TEXTURE1_ARB,-1,-1);
			glVertex2f(80,80);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1,-1);
			glMultiTexCoord2fARB(GL_TEXTURE1_ARB,1,-1);
			glVertex2f(-80,80);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1,1);
			glMultiTexCoord2fARB(GL_TEXTURE1_ARB,1,1);
			glVertex2f(-80,-80);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB,-1,1);
			glMultiTexCoord2fARB(GL_TEXTURE1_ARB,-1,1);
			glVertex2f(80,-80);
		glEnd();
	glEndList();

	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);

	return;
}

static void draw()
{
	static GLfloat alpha = 0, scale_phase = 0;
	GLfloat sine, scale;

	sine = (GLfloat)(sin(scale_phase) * 0.5);

	glMatrixMode(GL_TEXTURE);
	
	glActiveTextureARB(GL_TEXTURE0_ARB);
	glLoadIdentity();
	scale = 1 + sine;
	glScalef(scale,scale,scale);
	glRotatef(alpha,0,0,1);
	
	glActiveTextureARB(GL_TEXTURE1_ARB);
	glLoadIdentity();
	scale = 1 - sine;
	glScalef(scale,scale,scale);
	glRotatef(-alpha / 5,0,0,1);

	glMatrixMode(GL_MODELVIEW);

	glClear(GL_COLOR_BUFFER_BIT);

	glCallList(quad);

	alpha += 1;
	scale_phase += (1.0 / 180.0);
}

static void
draw_shutdown(void)
{
	glDeleteLists(quad, 1);
}

/* debug */

V_LOCAL vgl_context static_context = { NULL };

vgl_context *vglGetCurrentContext(void)
{
	return &static_context;
}

#define	WIDTH	320
#define	HEIGHT	240

int 
main(void)
{
	char *classes[] = {"no_event", "key_press", "key_release", "pointer_motion"};
	event_class_t ev_cls;
	int code, chr, px, py;
	if (!ref_init("Separate texture matrices", WIDTH, HEIGHT, 0, 0, 0, 0) || !vglGetVersionString())
		return -1;
	if(!HAVE_EXT(GL_ARB_multitexture))
	{
		puts("This demo requires GL_ARB_multitexture");
		return -1;
	}
	draw_setup(WIDTH, HEIGHT);
	do {
		draw();
		ref_end_frame();
		ev_cls = ref_next_event(&code, &chr, &px, &py);
		if (ev_cls == no_event) {
			ref_delay(10);	/* limit idle FPS to 100 */
			continue;
		}
		printf("[%s]: CODE '%i', CHAR '%i' ('%c'), PX %i, PY %i\n",
			   classes[ev_cls], code, chr,
			   (chr >= 32) ? chr : ' ',
			   px, py);
	} while (code != key_escape);
	draw_shutdown();
	ref_shutdown();
	return 0;
}
