#ifdef __cplusplus
#define	VECLIB_INLINE inline
#else
#define	VECLIB_INLINE __inline
#endif

#include <math.h>

typedef GLfloat vec_t;
typedef vec_t	vec3_t[3];

VECLIB_INLINE void
VectorSet(vec3_t dst,vec_t x,vec_t y,vec_t z)
{
	dst[0] = x;
	dst[1] = y;
	dst[2] = z;
}

VECLIB_INLINE void
VectorCopy(vec3_t dst,vec3_t src)
{
	dst[0] = src[0];
	dst[1] = src[1];
	dst[2] = src[2];
}

VECLIB_INLINE void
VectorAdd(vec3_t dst,vec3_t vc2)
{
	dst[0] += vc2[0];
	dst[1] += vc2[1];
	dst[2] += vc2[2];
}

VECLIB_INLINE void
VectorScale(vec3_t dst,vec_t v)
{
	dst[0] *= v;
	dst[1] *= v;
	dst[2] *= v;
}

VECLIB_INLINE GLfloat
DotProduct(vec3_t v1,vec3_t v2)
{
	return v1[0] * v2[0] + v1[1] * v2[1] + v1[2] * v2[2];
}

VECLIB_INLINE void
VectorNormalize(vec3_t vc)
{
	vec_t len = DotProduct(vc,vc);
	if(len > 1e-12)
	{
		len = 1.0f / (vec_t)sqrt(len);
		VectorScale(vc, len);
	}
	else
		VectorSet(vc,0,0,1);
}
